@@ -2,7 +2,7 @@
 /**
  * This class contains functions related to handling the headers of MIME data.
  *
- * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -69,7 +69,7 @@ class Horde_Mime_Headers implements Serializable
     protected $_singleFields = array(
         // Mail: RFC 5322
         'to', 'from', 'cc', 'bcc', 'date', 'sender', 'reply-to',
-        'message-id', 'in-reply-to', 'references', 'subject', 'x-priority',
+        'message-id', 'in-reply-to', 'references', 'subject',
         // MIME: RFC 1864
         'content-md5',
         // MIME: RFC 2045
@@ -88,7 +88,12 @@ class Horde_Mime_Headers implements Serializable
         // MIME: RFC 3282
         'content-language',
         // MIME: RFC 3297
-        'content-alternative'
+        'content-alternative',
+        // Importance: See, e.g., RFC 4356 [2.1.3.3.1]
+        'importance',
+        // OTHER: X-Priority
+        // See: http://kb.mozillazine.org/Emulate_Microsoft_email_clients
+        'x-priority'
     );
 
     /**
@@ -632,10 +637,9 @@ class Horde_Mime_Headers implements Serializable
 
     /**
      * Builds a Horde_Mime_Headers object from header text.
-     * This function can be called statically:
-     *   $headers = Horde_Mime_Headers::parseHeaders().
      *
-     * @param string $text  A text string containing the headers.
+     * @param mixed $text  A text string (or, as of 2.3.0, a Horde_Stream
+     *                     object or stream resource) containing the headers.
      *
      * @return Horde_Mime_Headers  A new Horde_Mime_Headers object.
      */
@@ -645,9 +649,16 @@ class Horde_Mime_Headers implements Serializable
         $mime = self::mimeParamFields();
         $to_process = array();
 
-        foreach (explode("\n", $text) as $val) {
-            $val = rtrim($val, "\r\n");
-            if (empty($val)) {
+        if ($text instanceof Horde_Stream) {
+            $stream = $text;
+            $stream->rewind();
+        } else {
+            $stream = new Horde_Stream_Temp();
+            $stream->add($text, true);
+        }
+
+        while (!$stream->eof()) {
+            if (!($val = rtrim($stream->getToChar("\n", false), "\r"))) {
                 break;
             }
 
